;--------------------------------------------------------------
; Filtering LF data
;
; Author : Fuminori Tsuchiya (Tohoku-U, Japan)
;
; Date   : 2015-01-23
;
; (in) data_in   : input 1D-data array
;      mask      : data mask pattern (0:mask)
;      width     : window size of filter
;      set_median, set_smooth, or set_average :
;                  filter to be applied
; (out) data_out : output data array
;       number of element in data_out is the same as data_in for set_median and set_smooth
;       number of element in data_out is reduced to be n_element(data_in)/width for set_average
;
; Example)
;    IDL> lf_filter, data_in=lf_pha[1,*], width=10, mask=mask, data_out=data_out, /set_average
;
;--------------------------------------------------------------
pro lf_filter, data_in=data_in, data_out=data_out, mask=mask, width=width, set_median=set_median, set_smooth=set_smooth, set_average=set_average
;
  if ~keyword_set(set_median) then set_median=0
  if ~keyword_set(set_smooth) then set_smooth=0
  if ~keyword_set(set_average) then set_average=0
  
; ----------
; ckeck keyword
; ----------
  if ( (set_median + set_smooth + set_average) ne 1 ) then begin
    print, 'LF_FILTER : One keyword should be specified.'
    return
  endif

  if ( ~keyword_set(width) or ~keyword_set(data_in) ) then begin
    print, 'LF_FILTER : Keywords width or data_in are not found.'
    return
  endif

; ----------
;  pre-processing
; ----------
  nd = n_elements(data_in)
  data_out = fltarr(nd)
  nw = width/2L

  data_work = data_in
  ; set missing value
  if keyword_set(mask) then begin
    for i=0L,nd-1L do begin
      if mask[i] eq 0 then data_work[i] = !values.f_nan
    endfor
  endif

; ----------
;  median
; ----------
  if set_median eq 1 then begin
    for i = nw, nd-(nw+1) do begin
      data_out[i] = median(data_work[i-nw:i+nw])
    endfor
  endif
    
; ----------
;  smooth
; ----------
  if set_smooth eq 1 then begin
    data_out = smooth(data_work, width, /nan)
  endif

; ----------
;  average
; ----------
  if set_average eq 1 then begin
    j=0L
    for i = 0L, nd-1L, width do begin
      data_out[j] = mean(data_work[i:i+width-1],/nan)
      j = j + 1
    endfor
  endif

end