;--------------------------------------------------------------
; get lighting location in specified time range (from sta_date to end_date)
;
; Author : Fuminori Tsuchiya (Tohoku-U, Japan)
;
; Date   : 2015-01-23
;
; (in)  dir      : directory of WWLLN data
;       date     : date 'yyyymmdd' [UT]
;       sta_date : start time 'hh:nn:ss.sss' [UT]
;       end_date : end   tmie 'hh:nn:ss.sss' [UT]
;       lon_range: longitude range [deg]
;       lat_range: latitude range [deg]
; (out) ltime    : time of lighting 'hh:nn:ss.sss' [UT]
;       llon     : longitude of lighting [degree]
;       llat     : latitude of lighting [degree]
;
;--------------------------------------------------------------
pro lf_get_wwlln, dir=dir, date=date, sta_time=sta_time, end_time=end_time, $
                  lon_range=lon_range, lat_range=lat_range, $
                  ltime=ltime, llon=llon, llat=llat
;
;----------------
; Check inputs
;----------------
if ( ~keyword_set(dir) or ~keyword_set(date) or ~keyword_set(sta_time) or ~keyword_set(end_time) ) then begin
  print, 'usage : lf_get_wwlln, dir=dir, date=date, sta_time=sta_time, end_time=end_time, llon=llon, llat=llat'
  return
endif
if ~keyword_set(lon_range) then lon_range=[0.0,360.0]
if ~keyword_set(lat_range) then lat_range=[-90.0,90.0]

sta_ut = strsplit(sta_time, ':', /extract)
sta_sec = sta_ut[0]*3600.0 + sta_ut[1]*60.0 + sta_ut[2]

end_ut = strsplit(end_time, ':', /extract)
end_sec = end_ut[0]*3600.0 + end_ut[1]*60.0 + end_ut[2]

;----------------
; Data file name and size
;----------------
file_in = dir+'A'+date+'.loc'
nd = file_lines (file_in)
  
;----------------
; open data file
;----------------
print, 'Reading ',file_in

openr, lun, file_in, /get_lun

cline = ''
i_save = 0
for i=0L,nd-1L do begin
  readf, lun, cline
  vals = strsplit(cline, ',', /extract)

  l_ut = strsplit(vals[1], ':', /extract)
  l_sec = l_ut[0]*3600.0 + l_ut[1]*60.0 + l_ut[2]

  if ( (l_sec ge sta_sec) and (l_sec le end_sec) and $
       (vals[2] ge lat_range[0]) and (vals[2] lt lat_range[1]) and $
       (vals[3] ge lon_range[0]) and (vals[3] lt lon_range[1]) ) then begin
    if i_save eq 0 then begin
      ltime = strarr(1)
      llat  = dblarr(1)
      llon  = dblarr(1)
      ltime[0] = vals[1]
      llat[0]  = double(vals[2])
      llon[0]  = double(vals[3])
    endif else begin
      ltime = [ltime, vals[1]]
      llat  = [llat,  double(vals[2])]
      llon  = [llon,  double(vals[3])]
    endelse
    i_save ++
  endif

endfor

free_lun, lun

print, 'lf_get_wwlln : number of lightning = ',i_save

end
