;------------------------------------------------------------------------------------------------------- 
; Load procedure to read LF amplitude and phase data
; need search_jjy_code.pro
; Ver. 1.0  2015-01-09  F. Tsuchiya
;------------------------------------------------------------------------------------------------------- 
pro read_lfdata, dir=dir, file=file, $
                 rx=rx, date=date, tx_fq_read=tx_fq_read, $ 
                 time_corr=time_corr, $
                 n_arr=n_arr,lf_time=lf_time, lf_amp=lf_amp, lf_pha=lf_pha, $
                 jjy_code=jjy_code, $
                 year=year,month=month,day=day,hour=hour,f_samp=f_samp,n_fft=n_fft,n_fq=n_fq,n_block=n_block, $
                 tx_fq=tx_fq
;
;  (in)     dir        : directory of input data file
;           file       : input file name
;           rx         : receiver station name
;           date       : date and time of data file (yyyymmddhh)
;           tx_fq_read : frequency channel (kHz) to read
;                        if this keyword does not set, this procedure will read only header
;           time_corr  : if set, lf_time is corrected using JJY time code (Valid for Japanese RX station)
;  (out)    n_arr      : number of output data point
;           lf_time    : second of the day
;           lf_amp     : amplitude of LF data (dB)
;           jjy_code   : JJY time code (Valid for Japanese RX station)
;           year       : year of data (YYYY)
;           month      : month
;           day        : day
;           hour       : hour
;           f_samp     : sampling frequency [kHz]
;           n_fft      : number of FFT point
;           n_fq       : number of TX frequencies recorded
;           n_block    : data blcock size (amount of 1-sec data, byte)
;           tx_fq      : list of TX frequencies recorded (kHz) 
;
; RX list available for this procedure 
;          ATH
;          NYA
;          PKR
;          ZAO
;          TWN
;          PTK
;          SBR
;          SGR
;          TKN
;
; Example ( TKN2015010100.dat in C:\data\ )
; IDL> dir = 'C:\data\'
; IDL> read_lfdata, dir=dir, date='2015010100', rx='TKN', tx_fq_read=[40.0,60.0], $
;                   lf_time=lf_time, lf_amp=lf_amp, lf_pha=lf_pha, $
;                   jjy_code=jjy_code, /time_corr
;   Reading C:\data\TKN2015010100.dat
;   Generate JJY time code
;   2015/001 00:00:00.2
;   lf_time has been synchronized with JJY time code
;   Done... Number of data:       36000
;
; IDL> plot, lf_time, lf_amp[0,*] ,xrange=[0,100], title='40kHz', xtitle='Time [sec]', ytitle='Amplitude [dB]'
; IDL> plot, lf_time, lf_pha[1,*] ,xrange=[0,100], title='60kHz', xtitle='Time [sec]', ytitle='Phase [degree]'
; IDL> plot, lf_time, jjy_code ,xr=[55,65], yr=[-0.1,1.5], xtitle='Time [sec]', ytitle='JJY time code'
;------------------------------------------------------------------------------------------------------- 

;----------------
; data file name
;---------------- 
;dir  = 'C:\cygwin\home\ft\LF\idl\'
if ( keyword_set(file) )then begin
  file_in = dir + file
endif else if ( keyword_set(date) and keyword_set(rx) )then begin
  file_in = dir + rx + date + '.dat'
endif else begin
  print, 'set directory and data file (of RX/date ) name'
  print, '> read_lfdata, dir="C:\data\", file="TKN2015010100.dat"'
  print, '> read_lfdata, dir="C:\data\", rx="TKN", date = "2015010100"'
  return
endelse

print, 'Reading ',file_in

;----------------
; open data file
;----------------
openr, lun, file_in, /get_lun
file_info = fstat(lun)

;----------------
; read header
;----------------
header = intarr(7)
readu, lun, header
year    = header(0)
month   = header(1)/100
day     = header(1) mod 100
hour    = header(2)
f_samp  = header(3)
n_fft   = header(4)
n_fq    = header(5)
n_block = header(6)

; read header (TX freqneucies)
tx_fq_int = intarr(n_fq)
tx_fq     = fltarr(n_fq)
readu, lun, tx_fq_int
tx_fq = float(tx_fq_int)/100

;print, header
;print, tx_fq

; read header (zero-padding area)
header = intarr(n_block/2 - 7 - n_fq)
readu, lun, header

if ~keyword_set (tx_fq_read) then begin
  return
endif

;----------------
; set JJY time code source
;----------------
jjy_fq = 0.0
if keyword_set(rx) then begin
  case rx of
  'TKN'  :  jjy_fq = 40.0
  'ZAO'  :  jjy_fq = 40.0
  'SGR'  :  jjy_fq = 60.0
  else   :  jjy_fq = 0.0
  endcase
endif

i_jjy = -1
if (jjy_fq ne 0.0) then begin
  for j=0,n_fq-1 do begin
    if (jjy_fq eq tx_fq[j]) then i_jjy = j
  endfor
endif

;----------------
; check position of data
;----------------
n_fq_read = n_elements(tx_fq_read)
i_fq = intarr(n_fq_read)
for i=0,n_fq_read-1 do begin
  i_fq[i]=-1
  for j=0,n_fq-1 do begin
    if (tx_fq_read[i] eq tx_fq[j]) then i_fq[i]=j
  endfor
  if (i_fq[i] eq -1) then begin
    print, '   Invalid set of tx_freq :',tx_fq_read[i]
    n_arr = 0
    return
  endif
endfor

;----------------
; read data
;----------------
n_blk = (file_info.size / n_block - 1)
n_arr = n_blk * 10L
lf_time  = fltarr(n_arr)
lf_amp   = fltarr(n_fq_read, n_arr)
lf_pha   = fltarr(n_fq_read, n_arr)

jjy_amp  = fltarr(n_arr)
jjy_code = bytarr(n_arr)

data = intarr(n_block/2)
for i=0,n_blk-1 do begin
  readu, lun, data

  if (data[0] ne 32767) then begin
    print, '   Invalid structure in data file ', file_in
    print, '      Start record is not 32767 at block ',i+1
    n_arr = 0
    return
  endif

  minute = data[1] / 100
  second  = data[1] mod 100

  for j=0,9 do begin
    ind = i*10L + j
    lf_time[ind] = hour*3600L + minute*60L + second*1L + j*0.1

    ofs = 2 + j*n_fq*2
    for k = 0, n_fq_read[0] - 1 do begin
      lf_amp[k,ind] = data[ofs + i_fq[k]] / 100.0                        ;[dB]
      lf_pha[k,ind] = data[ofs + n_fq + i_fq[k]] / 1000.0 * 180.0 / !pi  ;[radian -> degree]
    endfor
    if (i_jjy ne -1) then begin
      jjy_amp[ind] = data[ofs + i_jjy] / 100.0
    endif
    
  endfor

;  print, data[0],data[1]

endfor

free_lun, lun

;----------------
; JJY time code
;----------------
iwnd = 20
if (i_jjy ne -1) then begin

  for i=0L,n_arr-1L do begin
    jmin = max([i-iwnd,0])
    jmax = min([i+iwnd,n_arr-1])
    ave = mean(jjy_amp[jmin:jmax])
    if (jjy_amp[i] gt ave) then begin
      jjy_code[i] = 1
    endif else begin
      jjy_code[i] = 0
    endelse
  endfor
  
  print, '   Generate JJY time code'

  lf_search_jjy_code, jjy_code[0:1200], decode_time
  if (decode_time[0] eq 0) then begin
    print, '   Could not found time code'
  endif else begin
    print, decode_time[1],decode_time[2],decode_time[3],decode_time[4],decode_time[5],decode_time[6], $
           format='("   ",i4,"/",i03," ",i02,":",i02,":",i02,".",i1)'
  endelse
  
  if keyword_set(time_corr) then begin
    start_time = decode_time[3]*3600.0 + decode_time[4]*60.0 + decode_time[5] + decode_time[6]*0.1
    for i=0L, n_arr-1L do begin
      lf_time[i] = start_time + i*0.1
    endfor
    print, '   lf_time has been synchronized with JJY time code'
  endif
endif

print, 'Done... Number of data:',n_arr


end
