;------------------------------------------------------------------------------------------------------- 
; Load procedure to read LF amplitude and phase data (for Ver1.X format data)
; need search_jjy_code.pro
; Ver. 1.0  2015-01-09  F. Tsuchiya
;
;  (in)     dir        : directory of input data file
;           rx         : receiver station name
;           date       : date and time of data file (yyyymmddhh)
;           tx_fq_read : frequency channel (kHz) to read
;                        if this keyword does not set, this procedure will read only header
;           time_corr  : if set, lf_time is corrected using JJY time code (Valid for Japanese RX station)
;           lf_sver0    : if set, ver1.0 data format (no time record in data file). if not set, ver1.1 data format
;  (out)    n_arr      : number of output data point
;           lf_time    : second of the day
;           lf_amp     : amplitude of LF data (dB)
;           jjy_code   : JJY time code (Valid for Japanese RX station)
;           n_fq       : number of TX frequencies recorded
;           tx_fq      : list of TX frequencies recorded (kHz) 
;
; RX list available for this procedure 
;          RKB
;          ZAO
;          TWN
;
; Example ( twn2014111110.dat in C:\_LF\ver1\ )
; IDL> dir = 'C:\_LF\ver1\'
; IDL> read_lfdata_v1, dir=dir, date='2014111110', rx='twn', tx_fq_read=[40.0,60.0], $
;                   lf_time=lf_time, lf_amp=lf_amp, lf_pha=lf_pha, $
;                   jjy_code=jjy_code, /time_corr
;
; IDL> plot, lf_time, lf_amp[0,*] ,xrange=[0,100], title='40kHz', xtitle='Time [sec]', ytitle='Amplitude [dB]'
; IDL> plot, lf_time, lf_pha[1,*] ,xrange=[0,100], title='60kHz', xtitle='Time [sec]', ytitle='Phase [degree]'
;------------------------------------------------------------------------------------------------------- 
;
pro read_lfdata_v1, dir=dir, rx=rx, date=date, tx_fq_read=tx_fq_read, $ 
                 time_corr=time_corr, $
                 lf_sver0=lf_sver0, $
                 n_arr=n_arr,lf_time=lf_time, lf_amp=lf_amp, lf_pha=lf_pha, $
                 jjy_code=jjy_code, $
                 n_fq=n_fq, tx_fq=tx_fq

;----------------
; data file name
;---------------- 
if ( keyword_set(lf_sver0) ) then begin
  lf_ver = 0
endif else begin
  lf_ver = 1
endelse

if ( keyword_set(date) and keyword_set(rx) )then begin
  file_in = dir + rx + date + '.dat'
  if lf_ver eq 0 then file_in = file_in + '.0'
endif else begin
  print, 'set directory and data file (of RX/date ) name'
  print, '> read_lfdata_v1, dir="C:\data\", rx="twn", date = "2015010100"'
  return
endelse

lf_hr = fix(strmid(date,6,2))

;----------------
; TX frequency info of each RX station
;----------------
case rx of
'twn' : begin
        tx_fq = [40.00, 60.00, 19.80, 21.40, 22.20, 24.80, 25.00, 50.00, 54.00, 68.50]
        i_code = [0,1]
        end
'rkb' : begin
        tx_fq = [40.00, 60.00, 19.80, 21.40, 22.20, 24.80, 25.00, 50.00, 54.00, 68.50]
        i_code = [0,1]
        end
'zao' : begin
        tx_fq = [40.00, 60.00, 19.80, 21.40, 22.20, 24.80, 25.00, 50.00, 54.00, 68.50]
        i_code = [0]
        end
endcase
n_fq = n_elements(tx_fq)
n_code = n_elements(i_code)

;----------------
; check position of data
;----------------
n_fq_read = n_elements(tx_fq_read)
i_fq = intarr(n_fq_read)
for i=0,n_fq_read-1 do begin
  i_fq[i]=-1
  for j=0,n_fq-1 do begin
    if (tx_fq_read[i] eq tx_fq[j]) then i_fq[i]=j
  endfor
  if (i_fq[i] eq -1) then begin
    print, '   Invalid set of tx_freq :',tx_fq_read[i]
    n_arr = 0
    return
  endif
endfor

;----------------
; open data file
;----------------
print, 'Reading ',file_in

openr, lun, file_in, /get_lun
n_arr = file_lines (file_in)
if n_arr gt 0 then begin
  lf_time = fltarr(n_arr)
  lf_amp  = fltarr(n_fq_read, n_arr)
  lf_pha  = fltarr(n_fq_read, n_arr)
  jjy_amp = fltarr(n_arr)
  jjy_code = bytarr(n_arr)
endif else begin
  return
endelse

cline = ''
for i=0L,n_arr-1L do begin
  readf, lun, cline
  vals = strsplit(cline, ' ', /extract)
  
  if (lf_ver eq 0) then begin
    lf_time[i] = float(lf_hr)*3600.0 + float(i)*0.1
  endif else begin 
    lf_time[i] = vals[0]
  endelse
  for j=0,n_fq_read-1 do begin
    lf_amp[j,i] = vals[i_fq[j]*2+lf_ver]
    lf_pha[j,i] = vals[i_fq[j]*2+lf_ver+1]
  endfor

  for k=0,n_code-1 do begin
    jjy_amp[i] += 10^(vals[i_code[k]*2+lf_ver]*0.1)
  endfor
  jjy_amp[i] = 10.0*alog10(jjy_amp[i])

endfor

free_lun, lun

;----------------
; JJY time code
;----------------
iwnd = 20

for i=0L,n_arr-1L do begin
  jmin = max([i-iwnd,0])
  jmax = min([i+iwnd,n_arr-1])
  ave = mean(jjy_amp[jmin:jmax])
  if (jjy_amp[i] gt ave) then begin
    jjy_code[i] = 1
  endif else begin
    jjy_code[i] = 0
  endelse
endfor
  
print, '   Generate JJY time code'

lf_search_jjy_code, jjy_code[0:1200], decode_time
if (decode_time[0] eq 0) then begin
  print, '   Could not found time code'
endif else begin
  print, decode_time[1],decode_time[2],decode_time[3],decode_time[4],decode_time[5],decode_time[6], $
         format='("   ",i4,"/",i03," ",i02,":",i02,":",i02,".",i1)'
endelse
  
if keyword_set(time_corr) then begin
  start_time = decode_time[3]*3600.0 + decode_time[4]*60.0 + decode_time[5] + decode_time[6]*0.1
  for i=0L, n_arr-1L do begin
    lf_time[i] = start_time + i*0.1
  endfor
  print, '   lf_time has been synchronized with JJY time code'
endif

print, 'Done... Number of data:',n_arr

end
