;------------------------------------------------------------------------------------------------------- 
; Load procedure to read LF specrea (amplitude and phase) data
; Ver. 1.0  2015-01-09  F. Tsuchiya
;------------------------------------------------------------------------------------------------------- 
;
;  (in)     dir        : directory of input data file
;           file       : input file name
;           rx         : receiver station name
;           date       : date and time of data file (yyyymmddhh)
;           tx_fq_read : frequency channel (kHz) to read
;                        if this keyword does not set, this procedure will read only header
;  (out)    n_arr      : number of output data point (time)
;           n_arr      : number of output data point (freq)
;           lf_spc_amp : 2d array of lf amplitude dynamic spectrum
;           lf_spc_pha : 2d array of lf phase dynamic spectrum
;           lf_time    : second of the day (m_arr)
;           lf_fq      : frequency (n_arr)
;           lf_amp     : amplitude of LF data at frequencies given by tx_fq_read
;           lf_pha     : phase of LF data at frequencies given by tx_fq_read
;           year       : year of data (YYYY)
;           month      : month
;           day        : day
;           hour       : hour
;           f_samp     : sampling frequency [kHz]
;           n_fft      : number of FFT point
;           d_tm       : time resolution [sec]
;           d_fq       : frequency resolution [Hz]
;           n_block    : data blcock size (amount of 1-sec data, byte)
;
; RX list available for this procedure 
;          ATH
;          NYA
;          PKR
;          ZAO
;          TWN
;          PTK
;          SBR
;          SGR
;          TKN
;
; Example ( TKN2015010100.dat in C:\data\ )
; IDL> dir = 'C:\data\'
; IDL> read_lfspec, dir=dir, date='2015010100', rx='TKN', tx_fq_read=[40.0,60.0,22,2], $
;      lf_time=lf_time, lf_fq=lf_fq, lf_amp=lf_amp, lf_pha=lf_pha, lf_spc_amp=lf_spc_amp
;
; IDL> plot, lf_time, lf_amp[0,*] ,xrange=[0,3600], title='40kHz', xtitle='Time [sec]', ytitle='Amplitude [dB]'
; IDL> plot, lf_time, lf_pha[1,*] ,xrange=[0,3600], title='60kHz', xtitle='Time [sec]', ytitle='Phase [degree]'
; IDL> plot, lf_time, lf_amp[2,*] ,xrange=[0,3600], title='22.2kHz', xtitle='Time [sec]', ytitle='Amplitude [dB]'
; IDL> img_plot, lf_spc_amp,lf_time,lf_fq
;------------------------------------------------------------------------------------------------------- 
;
pro read_lfspec, dir=dir, file=file, rx=rx, date=date, tx_fq_read=tx_fq_read, $
                 m_arr=m_arr, n_arr=n_arr, lf_spc_amp=lf_spc_amp, lf_spc_pha=lf_spc_pha, $
                 lf_time=lf_time, lf_fq=lf_fq, lf_amp=lf_amp, lf_pha=lf_pha, $
                 year=year,month=month,day=day,hour=hour,f_samp=f_samp,n_fft=n_fft,d_tm=d_tm,d_fq=d_fq,n_block=n_block

;----------------
; data file name
;---------------- 
;dir  = 'C:\cygwin\home\ft\LF\idl\'
if ( keyword_set(file) )then begin
  file_in = dir + file
endif else if ( keyword_set(date) and keyword_set(rx) )then begin
  file_in = dir + rx + date + '.spc'
endif else begin
  print, 'set directory and data file (of RX/date ) name'
  print, '> read_lfspec, dir="C:\data\", file="TKN2015010100.spc"'
  print, '> read_lfspec, dir="C:\data\", rx="TKN", date = "2015010100"'
endelse

print, 'Reading ',file_in

;----------------
; open data file
;----------------
openr, lun, file_in, /get_lun
file_info = fstat(lun)

;----------------
; read header
;----------------
header = intarr(10)
readu, lun, header
year    = header(0)
month   = header(1)/100
day     = header(1) mod 100
hour    = header(2)
f_samp  = header(3)
n_fft   = header(4)
d_tm    = header(5)
n_ave_fq = header(6)
n_arr   = header(7)
d_fq    = header(8)
n_block = header(9)

print, header

; read header (zero-padding area)
header = intarr(n_block/2 - 10)
readu, lun, header

;----------------
; set frequency
;----------------
lf_fq = d_fq * findgen(n_arr) * 0.001  ; kHz

;----------------
; check position of data
;----------------
n_fq_read = n_elements(tx_fq_read)
i_fq = fix(tx_fq_read * 1000 / d_fq)

;----------------
; read data
;----------------
m_arr = (file_info.size / n_block - 1)
lf_time  = fltarr(m_arr)
lf_amp   = fltarr(n_fq_read, m_arr)
lf_pha   = fltarr(n_fq_read, m_arr)

lf_spc_amp  = fltarr(m_arr, n_arr)
lf_spc_pha  = fltarr(m_arr, n_arr)

data = intarr(n_block/2)
for i=0,m_arr-1 do begin
  readu, lun, data

  if (data[0] ne 32767) then begin
    print, '   Invalid structure in data file ', file_in
    print, '      Start record is not 32767 at block ',i+1
    m_arr = 0
    return
  endif

  minute = data[1] / 100
  second  = data[1] mod 100
  lf_time[i] = hour*3600.0 + minute*60.0 + second

  for j=0,n_arr-1 do begin
    lf_spc_amp[i,j] = data[2+j] / 100.0                       ;[dB]
    lf_spc_pha[i,j] = data[2+n_arr+j] / 1000.0 * 180.0 / !pi  ;[radian -> degree]
  endfor
  for j = 0, n_fq_read[0] - 1 do begin
    lf_amp[j,i] = lf_spc_amp[i,i_fq[j]]
    lf_pha[j,i] = lf_spc_pha[i,i_fq[j]]
  endfor

;  print, data[0],data[1]

endfor

free_lun, lun

print, 'Done... Number of data:', m_arr, n_arr


end
